//*****************************************************************************
// Hello_world.c - teste de uso da Linguagem C com compilador SDCC e placa
//                 Z80 1990 by Arne
//
// Desenvolvido: Eng. Márcio José Soares
// Versão      : 1.0 - 27/03/2023
// Compilador  : SDCC v4.0.0.0
// Plataforma  : Placa Z80 1990 by Arne
//
//*****************************************************************************
//
// Últimas alterações:
// 	em 27/03/2023:
//		- criado e testado esse programa
//*****************************************************************************

//*****************************************************************************
// Arquivos incluídos no módulo
//*****************************************************************************
#include <stdio.h>
#include <string.h>

//*****************************************************************************
// Definições importantes do módulo
//*****************************************************************************
#define TRUE 		0x01
#define FALSE 		0x00

#define byte 		unsigned char

#define HALT 		__asm halt __endasm // aguarda pela próxima int

//*****************************************************************************
// Definições dos endereços das subrotinas do monitor da placa Z80 by Arne
//*****************************************************************************
#define W8255A		0x88				// subrotina acesso port A do 8255
#define W8255B		0x85				// subrotina acesso port B do 8255
#define R8255C		0x8B				// subrotina acesso port C do 8255
#define T500ms  	0xA9				// subrotina de espera 500ms
#define WRZERODISP	0x7C				// subrotina para zerar o display				
#define WRA_DISP56	0x7F				// subrotina p/ escrever A disps 5 e 6

//*****************************************************************************
// Definições dos endereços das variáveis do monitor da placa Z80 by Arne
//*****************************************************************************
#define MYDATA		0x87CE				// endereço da var mydata onde o monitor
										// insere a leitura do port C do 8255
					
//*****************************************************************************
// Declara Funções do módulo
//*****************************************************************************
void writeA8255(byte myA);
void wait500ms(void);
void zera_disp(void);
void senddisp56(byte myA);
byte readC8255(void);

//*****************************************************************************
// Funções
//*****************************************************************************

//*****************************************************************************
// Função principal - main
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*****************************************************************************
int main(void){
	
	byte aux = 0;
	byte aux1 = 0;
	
	zera_disp();						// inicia zerando o display
	
	while(TRUE){						// faz eternamente
		aux = readC8255();				// lê o port C
		senddisp56(aux);				// escreve no display 5 e 6
		writeA8255(aux1);				// envia 1 para o port A
		wait500ms();					// aguarda
		
		aux1 ^= 0x01;
	}
	
}

//*****************************************************************************
// Função para ler do portC do 8255
//
// retorno quando byte é feito em L  (8 bits)
// retorno quando int  é feito em HL (16 bits)
// retorno quando long é feito em DEHL (32 bits)
//
// Entradas - nenhuma
// Saídas   - byte recebido
//*****************************************************************************
byte readC8255(void) __naked {
	
	__asm
	
	call	#R8255C			; lê port C do 8255
	ld		hl,#MYDATA		; pega endereço da var 
	ld		a,(hl)			; em a o conteúdo
	ld		l,a             ; resposta quando byte sempre em l
	
	ret 
	
	__endasm;
	
}

//*****************************************************************************
// Função para escrever no portA do 8255
//
// Entradas - byte a enviar
// Saídas   - nenhuma
//*****************************************************************************
void writeA8255(byte myA) __naked {
	
	__asm
	
	ld		iy,#2		; inicia leitura do parâmetro
	add		iy,sp
	
	ld		a,(iy)		; pega valor e coloca em A

	call	#W8255A		; escreve valor no portA 
	ret
	
	__endasm;
	
}

//*****************************************************************************
// Função para escrever nos displays 5 e 6
//
// Entradas - valor hexa a escrever
// Saídas   - nenhuma
//*****************************************************************************
void senddisp56(char myA) __naked {
	
	__asm
	
	ld		iy,#2		; inicia leitura do parâmetro
	add		iy,sp
	
	ld		a,(iy)		; pega valor e coloca em A
	
	call	#WRA_DISP56	; envia A para o display
	ret
	
	__endasm;
	
}

//*****************************************************************************
// Função para zerar o display
//
// Entradas - memhuma
// Saídas   - nenhuma
//*****************************************************************************
void zera_disp(void) __naked {
	
	__asm
	
	call	#WRZERODISP  ;zera todos displays de 7 segmentos
	ret
	
	__endasm;
	
}

//*****************************************************************************
// Função para aguardar 500ms
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*****************************************************************************
void wait500ms(void) __naked {
	
	__asm
	
	call	#T500ms
	ret 
	
	__endasm;
	
}


